function [SIM_dyn,SIM_csd] = test_eval_dyn_pol()
% Evaluate Dynare policy function with "eval_dyn_pol_beta". Crucial part in 
% line 162-174.
%
% Dynare 4.6.4 used
%
% Solution compared to CSD toolbox (perturbation toolbox in folder CSD_v02.4)
%
% Model: standard RBC model. See folders "stnd_rbc_dyn" & "stnd_rbc_csd"

% Sijmen Duineveld, September 2021, s.a.duineveld@outlook.com

clc;
close all;

dbstop if error;

restoredefaultpath;
clear RESTOREDEFAULTPATH_EXECUTED;

%% Set parameters and get steady state
par.alpha       = 0.36;		%capital share income 
par.beta        = 0.985;	% discount factor
par.delta       = 0.025;	% depreciation of capital 
par.nu          = 2; 	% risk aversion 
par.eta         = 4; 	% Frisch elasticity of lab. supply
par.chi         = 1; 	% scalar for s.s hours worked

par.rho_z       = 0.95;	% autocorrelation coefficient TFP
par.sigma_z     = 0.01;	% standard dev. shocks in TFP 


%% Solve steady state
SS              = stnd_rbc_ss(par,1);
% the 1 is steady state TFP

% Settings for simulation
par.sim.TT      = 1000;%Number of periods per simulation
par.sim.cols    = 5;% Number of simulations 
par.sim.T_ini   = 10;% Initial number of periods at steady state

par.opt.order   = 3;% Order of the approximation (in simulation)


%% Solve with dynare, and simulate using 'eval_dyn_pol_beta'
run_dyn = 1;%Set to 0 to load saved results, set to 1 to recompute with Dynare

if run_dyn == 1
    % Add the folder with your dynare installation:
    addpath C:\dynare\4.6.4\matlab;

    save ('stnd_rbc_dyn\params','par','SS');

    cd stnd_rbc_dyn;
    
    dynare stnd_rbc_dyn noclearall;
    clearvars -except par SS;
    
    cd ..\;
end

load stnd_rbc_dyn\stnd_rbc_dyn_results oo_ M_;
DR              = oo_.dr;
DR.state_var    = M_.state_var;

% Simulate with dynare:
par.sim.sol_typ = 'dyn';
SIM_dyn = stnd_rbc_sim_pert(par,SS,DR,par.sim,par.opt.order);


%% Solve with CSD, and simulate using 'eval_sol_csd' :
run_csd = 1;%Set to 0 to load saved results, set to 1 to recompute with CSD toolbox

addpath 'CSD_v02.4';
addpath 'CSD_v02.4\subfun'; 
if run_csd == 1;
   
    addpath 'stnd_rbc_csd';
    
    [SOL] = solve_stnd_rbc_csd(par,SS,par.opt.order);
    
    save stnd_rbc_csd\CSD_sol SOL;
else
    load stnd_rbc_csd\CSD_sol SOL;
end

par.sim.sol_typ = 'csd';
SIM_csd = stnd_rbc_sim_pert(par,SS,SOL,par.sim,par.opt.order);


%% Compare differences
dZ = abs(SIM_csd.LZ - SIM_dyn.LZ);
dZ_max = max(dZ(:));

dK = abs(SIM_csd.LK - SIM_dyn.LK);
dK_max = max(dK(:));

dC = abs(SIM_csd.LC - SIM_dyn.LC);
dC_max = max(dC(:));

fprintf('\n\nMaximum differences between Dynare and CSD Toolbox in simulation\nlog(Z): %.2e; log(K): %.2e; log(C): %.3e\n',dZ_max,dK_max,dC_max);

end


%% SIMULATION given perturbation solution
function [SIM] = stnd_rbc_sim_pert(par,SS,DRSOL,opt_sim,order)
%[SIM] = stnd_rbc_sim_pert(par,SS,DRSOL,opt_sim,order)
%
%       DRSOL       = either Dynare decision rules (oo_.dr) when sol_typ = 'dyn' 
%                       or solution SOL  of CSD toolbox
%
%       opt_sim     = structure with settings for simulation. Contains:
%                       sol_typ - either 'dyn' or 'csd'
%                       TT      - number of periods in simulation (scalar)
%                       cls     - number of columns in simulation (scalar)
%                       T_ini   - initial periods at s.s.
%
%       order       = order of solution to be used
%
% OUTPUT:
%       SIM         = structure with series in logs: LK (capital), 
%                       LC (consumption), LH (hours worked), LZ (TFP), LY
%                       (output)

cols = opt_sim.cols;

% Initialize variables (LK, LC, LH, LZ,LH)
nms                 = {'K','C','H','Z','Y'}; 
for in = 1:size(nms,2)
   eval(['L',nms{in},' = NaN(opt_sim.T_ini+opt_sim.TT,cols);']); 
   eval(['L',nms{in},'(1:opt_sim.T_ini,:) = log(SS.',nms{in},'ss);']);  
end   
% pre-allocate one extra row for LK_t+1 and LZ_t+1
LK = [LK;NaN(1,cols)];
LZ = [LZ;NaN(1,cols)];

% initialize shocks:
rng('default');
epsilon = [zeros(cols,opt_sim.T_ini),randn(cols,opt_sim.TT+1)]';

% Set k_t+1 = k_ss 
LK(opt_sim.T_ini+1,:)  = log(SS.Kss);

% Set z_t+1 = lz(t)+eps(t+1)
if strcmp(opt_sim.sol_typ,'csd')
    LZ(opt_sim.T_ini+1,:) = get_ZZn(DRSOL,zeros(1,cols),epsilon(opt_sim.T_ini+1,:));
end

% loop over time (from T_ini +1 till T_ini + TT):
for it = opt_sim.T_ini + 1: opt_sim.T_ini + opt_sim.TT   
    
    if strcmp(opt_sim.sol_typ,'csd')
        % Get XXn, YY, ZZn using eval_sol_csd:
        [LK(it+1,:),YY_t,LZ(it+1,:)] = ...
            eval_sol_csd(DRSOL,LK(it,:),LZ(it,:),epsilon(it+1,:),order);
        
        LC(it,:) = YY_t(1,:);%Cons. is first control variable
        LH(it,:) = YY_t(2,:);%Hours is second control variable  
        LY(it,:) = YY_t(3,:);%Output is third control variable
    elseif strcmp(opt_sim.sol_typ,'dyn')
        Xp = [LK(it,:)',LZ(it-1,:)'];
        EPSt = epsilon(it,:)';
        
        [Xt,Yt] = eval_dyn_pol_beta(DRSOL,Xp,EPSt,order);
          
        % Transpose results (each simulation is column here, but a row in
        % output of eval_dyn_pol
        LK(it+1,:)  = Xt(:,1)';%K is 1st state variable in DR order
        LZ(it,:)    = Xt(:,2)';%Z is 2nd state variable in DR order
        
        LY(it,:) = Yt(:,1)';%Output is 1st control variable in DR order
        LC(it,:) = Yt(:,2)';%Cons. is 2nd control variable in DR order
        LH(it,:) = Yt(:,3)';%Hours is 3rd control variable in DR order        
    end
end

% remove last k_t+1 and z_t+1 (ie. adjust size to (T_ini+TT) x cols )
LK = LK(1:end-1,:);
LZ = LZ(1:end-1,:);


% Put variables in structure SIM:
for in = 1:size(nms,2)
   eval(['SIM.L',nms{in},' = L',nms{in},';']); 
end  

SIM.epsilon = epsilon;

end

%% Steady state
function [SS] = stnd_rbc_ss(par,Zss)
%[SS] = stnd_rbc_ss(par,Zss)
%
% Calculation of steady state of Standard RBC model 
%
% INPUTS:
%		par     = parameters of model
%
%       Zss     = steady state level of TFP

% Sijmen Duineveld, updated March 2021, s.a.duineveld@outlook.com

Omega = (1-par.beta*(1-par.delta))/ (par.alpha*par.beta*Zss);

Kss = ( ((1-par.alpha)/par.chi*Zss.*(Zss.*Omega-par.delta).^-par.nu ).^par.eta .* Omega.^((1+par.eta*par.alpha)/(par.alpha-1)) ).^(1/(1+par.eta*par.nu));

Hss = Omega.^(1/(1-par.alpha)) .* Kss;

Css = (Zss.*Omega - par.delta) .* Kss;

Yss = Zss *Kss^par.alpha*Hss^(1-par.alpha);

nms                 = {'K','C','H','Z','Y'}; 
for in = 1:size(nms,2)
   eval(['SS.',nms{in},'ss = ',nms{in},'ss;']);  
end

end

